// -*-C++-*-
// This file is part of the gmod package
// Copyright (C) 1997 by Andrew J. Robinson

/* This file is part of the GMOD package */

#ifndef __globalsH
#define __globalsH

extern int patternLen[MAX_POSITION];
extern int patternTempo[MAX_POSITION];
extern struct noteInfo *patternTable[MAX_PATTERN * MAX_TRACK];

extern struct voiceInfo voices[MAX_TRACK];

extern int tune[MAX_POSITION];
extern short voiceTable[MAX_POSITION][MAX_TRACK];
extern double tickDuration;
extern int  mixerfd;
extern double thisTime, nextTime;
extern int ticksPerDivision;
extern double clockRate;	/* HZ */

extern unsigned short periodTable[];

#ifndef USE_X
extern unsigned char stopFlag;
#endif
extern unsigned char background;
extern int actualPos;
extern int positionChange;

extern unsigned int seqInput;

#ifdef USE_X
extern struct xStruct xInfo;
extern int currentMod;
#endif
#endif
