// -*-C++-*-
// This file is part of the gmod package
// Copyright (C) 1997 by Andrew J. Robinson

#ifndef VOICEAWE_H
#define VOICEAWE_H

#include "Voice.h"

class VoiceAWE : public Voice
{
public:
  void doUpdates(int);

private:
  int doPreUpdates(int);
  void startNote(int, int);
};

#endif
