// -*-C++-*-
// This file is part of the gmod package
// Copyright (C) 1997 by Andrew J. Robinson

#ifndef VOICEGUS_H
#define VOICEGUS_H

#include "Voice.h"

class VoiceGUS : public Voice
{
public:
  void doUpdates(int);
};

#endif
