// -*-C++-*-
// This file is part of the gmod package
// Copyright (C) 1997 by Andrew J. Robinson

#ifndef OutBuffer_h
#define OutBuffer_h

struct OutBuffer
{
	int n_bytes;
	int offset;
	char buffer[2048];
};
#endif
