// -*-C++-*-
// This file is part of the gmod package
// Copyright (C) 1997 by Andrew J. Robinson

#include "defines.h"
#include "structs.h"
#include "globals.h"

void
initVoices ()
{
  int i;

  for (i = 0; i < MAX_TRACK; i++)
    {
      voices[i].pattern = 0;
      voices[i].position = 0;
      voices[i].loopTimes = 0;

      voices[i].lastInfo = 0;
    }
}
