// -*-C++-*-
// This file is part of the gmod package
// Copyright (C) 1997 by Andrew J. Robinson

#ifndef s3mH
#define s3mH

#include "Sample.h"

class S3M_sample : public Sample
{
public:
  int load(Sequencer &, FILE *, int, int, void *, void *);
  int slideRate(int, unsigned char) const;
};

inline int
S3M_sample::slideRate(int rate, unsigned char slideDir) const
{
   return ((int)(double(Sample::slideRate(rate, slideDir)) *
		 (baseFreq_ / 8363.0)));
}

#endif
