// -*-C++-*-
// This file is part of the gmod package
// Copyright (C) 1997 by Andrew J. Robinson

#ifndef __CommentShellH
#define __CommentShellH

#include <qapp.h>
#include <qmlined.h>
#include <qpushbt.h>

class CommentShell : public QWidget
{
  Q_OBJECT
public:
  CommentShell(QWidget *w = 0);
  void setComment(char *, int);

 public slots:
 void showCommentShell();

protected:
  void resizeEvent(QResizeEvent *);

private:
  QMultiLineEdit *commentEdit;
  QPushButton *closeButton;

 private slots:
 void closeCommentShell();
};


#endif
