// -*-C++-*-
// This file is part of the gmod package
// Copyright (C) 1997 by Andrew J. Robinson

#ifdef USE_LOCAL
#include "soundcard.h"
#else
#include <sys/soundcard.h>
#endif

#include "Sample.h"

void
Sample::getPatchInfo(struct patch_info &patch) const
{
  patch.mode = mode_;
  patch.len = length_;
  patch.loop_start = loopStart;
  patch.loop_end = loopEnd;
  patch.base_note = baseNote_;  /* set real base note here */
  patch.base_freq = baseFreq_;
}
